"use strict";
var request = require('request');
var log_1 = require("./../log");
var PrivateChannel = (function () {
    function PrivateChannel(options) {
        this.options = options;
        this.request = request;
    }
    PrivateChannel.prototype.authenticate = function (socket, data) {
        var options = {
            url: this.authHost() + this.options.authEndpoint,
            form: { channel_name: data.channel },
            headers: (data.auth && data.auth.headers) ? data.auth.headers : {},
            rejectUnauthorized: false
        };
        return this.severRequest(socket, options);
    };
    PrivateChannel.prototype.authHost = function () {
        return (this.options.authHost) ?
            this.options.authHost : this.options.host;
    };
    PrivateChannel.prototype.severRequest = function (socket, options) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            options.headers = _this.prepareHeaders(socket, options);
            _this.request.post(options, function (error, response, body, next) {
                if (!error && response.statusCode == 200) {
                    resolve(JSON.parse(response.body));
                }
                else {
                    log_1.Log.error(error);
                    reject('Could not send authentication request.');
                }
            });
        });
    };
    PrivateChannel.prototype.prepareHeaders = function (socket, options) {
        options.headers['Cookie'] = socket.request.headers.cookie;
        options.headers['X-Requested-With'] = 'XMLHttpRequest';
        return options.headers;
    };
    return PrivateChannel;
}());
exports.PrivateChannel = PrivateChannel;
