"use strict";
var fs = require('fs');
var http = require('http');
var https = require('https');
var io = require('socket.io');
var log_1 = require("./log");
var Server = (function () {
    function Server(options) {
        this.options = options;
    }
    Server.prototype.init = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.serverProtocol().then(function () {
                log_1.Log.success("Running at " + _this.options.host + " on port " + _this.options.port);
                resolve(_this.io);
            }, function (error) { return reject(error); });
        });
    };
    Server.prototype.serverProtocol = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (_this.options.protocol == 'https') {
                _this.secure().then(function () {
                    resolve(_this.httpServer(true));
                }, function (error) { return reject(error); });
            }
            else {
                resolve(_this.httpServer(false));
            }
        });
    };
    Server.prototype.secure = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.options.sslCertPath || !_this.options.sslKeyPath) {
                reject('SSL paths are missing in server config.');
            }
            Object.assign(_this.options, {
                cert: fs.readFileSync(_this.options.sslCertPath),
                key: fs.readFileSync(_this.options.sslKeyPath)
            });
            resolve(_this.options);
        });
    };
    Server.prototype.httpServer = function (secure) {
        if (secure) {
            this.http = https.createServer(this.options, this.httpHandler);
        }
        else {
            this.http = http.createServer(this.httpHandler);
        }
        this.http.listen(this.options.port, this.options.host);
        return this.io = io(this.http, this.options.socketio);
    };
    Server.prototype.httpHandler = function (req, res) {
        res.setHeader('Content-Type', 'application/json');
        res.setHeader('X-Powered-By', 'Laravel Echo Server');
    };
    return Server;
}());
exports.Server = Server;
