"use strict";
var log_1 = require("./../log");
var url = require('url');
var HttpSubscriber = (function () {
    function HttpSubscriber(options, http) {
        this.options = options;
        this.http = http;
    }
    HttpSubscriber.prototype.subscribe = function (callback) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.http.on('request', function (req, res) {
                var body = [];
                if (req.method == 'POST' && url.parse(req.url).pathname == '/broadcast') {
                    if (!_this.canAccess(req)) {
                        return _this.unauthorizedResponse(req, res);
                    }
                    res.on('error', function (error) {
                        log_1.Log.error(error);
                    });
                    req.on('data', function (chunk) { return body.push(chunk); })
                        .on('end', function () { return _this.handleData(req, res, body, callback); });
                }
                else {
                    if (url.parse(req.url).pathname != '/socket.io/') {
                        res.end();
                    }
                }
            });
            log_1.Log.success('Listening for http events...');
            resolve();
        });
    };
    HttpSubscriber.prototype.handleData = function (req, res, body, broadcast) {
        body = JSON.parse(Buffer.concat(body).toString());
        if (body.channel && body.message) {
            if (!broadcast(body.channel, body.message)) {
                return this.badResponse(req, res, "Could not broadcast to channel: " + body.channel);
            }
        }
        else {
            return this.badResponse(req, res, 'Event must include channel and message');
        }
        res.write(JSON.stringify({ message: 'ok' }));
        res.end();
    };
    HttpSubscriber.prototype.canAccess = function (req) {
        if (this.options.host == req.headers.referer) {
            return true;
        }
        var api_key = this.getApiToken(req);
        if (api_key) {
            var referrer = this.options.referrers.find(function (referrer) {
                return referrer.apiKey == api_key;
            });
            if (referrer && (referrer.host == '*' ||
                referrer.host == req.headers.referer)) {
                return true;
            }
        }
        return false;
    };
    HttpSubscriber.prototype.getApiToken = function (req) {
        if (req.headers.authorization) {
            return req.headers.authorization.replace('Bearer ', '');
        }
        if (url.parse(req.url, true).query.api_key) {
            return url.parse(req.url, true).query.api_key;
        }
        return false;
    };
    HttpSubscriber.prototype.unauthorizedResponse = function (req, res) {
        res.statusCode = 403;
        res.write(JSON.stringify({ error: 'Unauthorized' }));
        res.end();
        return false;
    };
    HttpSubscriber.prototype.badResponse = function (req, res, message) {
        res.statusCode = 400;
        res.write(JSON.stringify({ error: message }));
        res.end();
        return false;
    };
    return HttpSubscriber;
}());
exports.HttpSubscriber = HttpSubscriber;
